﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;
using BEPUphysics.Constraints;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// A long string of blocks connected by joints.
    /// </summary>
    public class BridgeDemo : StandardDemo
    {
        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Bridge"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public BridgeDemo(DemosGame game)
            : base(game)
        {
            //Form a long chain of cubes, connected by ball socket joints.

            float baseHeight = 85;
            float linkSeparation = 2f;
            int numLinks = 200;
            float xOffset = -(numLinks + 1) * linkSeparation / 2;
            Box link;
            Box previousLink = null;
            Entity ground = new Box(new Vector3(0, -3, 0), 10, 10, 10);
            space.add(ground);
            BallSocketJoint ballSocketJointToAdd;
            for (int k = 0; k < numLinks; k++)
            {
                link = new Box(new Vector3(xOffset + linkSeparation * (k + 1), baseHeight, 0), 1.1f, 1.1f, 1.1f, 10);
                space.add(link);
                if (k == 0)
                {
                    ballSocketJointToAdd = new BallSocketJoint(link, ground, new Vector3(xOffset, baseHeight, 0));
                    ballSocketJointToAdd.springSettings.stiffnessConstant = 90000;
                    ballSocketJointToAdd.springSettings.dampingConstant = 8000f;
                    space.add(ballSocketJointToAdd);
                }
                else if (k == numLinks - 1)
                {
                    ballSocketJointToAdd = new BallSocketJoint(link, previousLink, (link.centerPosition + previousLink.centerPosition) / 2);
                    ballSocketJointToAdd.springSettings.stiffnessConstant = 90000;
                    ballSocketJointToAdd.springSettings.dampingConstant = 8000f;
                    space.add(ballSocketJointToAdd);
                    ballSocketJointToAdd = new BallSocketJoint(link, ground, new Vector3(xOffset + (numLinks + 1) * linkSeparation, baseHeight, 0));
                    ballSocketJointToAdd.springSettings.stiffnessConstant = 90000;
                    ballSocketJointToAdd.springSettings.dampingConstant = 8000f;
                    space.add(ballSocketJointToAdd);
                }
                else
                {
                    ballSocketJointToAdd = new BallSocketJoint(link, previousLink, (link.centerPosition + previousLink.centerPosition) / 2);
                    ballSocketJointToAdd.springSettings.stiffnessConstant = 90000;
                    ballSocketJointToAdd.springSettings.dampingConstant = 8000f;
                    space.add(ballSocketJointToAdd);
                }
                previousLink = link;
            }


            game.camera.position = new Vector3(-180, 70, 300);
            game.camera.yaw = MathHelper.ToRadians(-24);
            game.camera.pitch = MathHelper.ToRadians(-5);
        }

    }
}
