﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// A large bunch of cubes suspended in space.
    /// </summary>
    public class BroadPhaseDemo : StandardDemo
    {
        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Broad phase Stress Test"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public BroadPhaseDemo(DemosGame game)
            : base(game)
        {
            //Make a fatter kapow sphere.
            space.remove(kapow);
            kapow = new Sphere(new Vector3(11000, 0, 0), 1f, 1000);
            kapow.collisionMargin = .5f;
            space.add(kapow);
            space.simulationSettings.collisionResponse.iterations = 1; //Essentially no sustained contacts, so don't need to worry about accuracy.
            space.simulationSettings.motionUpdate.gravity = Vector3.Zero;
            int numColumns = 15;
            int numRows = 15;
            int numHigh = 15;
            Entity toAdd;
            float separation = 3;
            for (int i = 0; i < numRows; i++)
                for (int j = 0; j < numColumns; j++)
                    for (int k = 0; k < numHigh; k++)
                    {


                        toAdd = new Box(new Vector3(separation * i, k * separation, separation * j), 1, 1, 1, 1);
                        toAdd.bounciness = 1; //Superbouncy boxes help propagate shock waves.
                        toAdd.angularDamping = 0f; //It looks cooler if boxes don't slowly stop spinning!
                        //toAddBox.isTangible = false;
                        space.add(toAdd);


                    }

            game.camera.position = new Vector3(0, 3, -10);
            game.camera.yaw = -(float)Math.PI;
        }

    }
}
