﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;
using BEPUphysics;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Blocks floating in a viscous fluid.
    /// </summary>
    public class BuoyancyDemo : StandardDemo
    {
        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Buoyancy"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public BuoyancyDemo(DemosGame game)
            : base(game)
        {
            List<Vector3[]> tris = new List<Vector3[]>();
            float basinWidth = 100;
            float basinLength = 100;
            float basinHeight = 16;
            float waterHeight = 15;

            //Remember, the triangles composing the surface need to be coplanar with the surface.  In this case, this means they have the same height.
            tris.Add(new Vector3[] { new Vector3(-basinWidth / 2, waterHeight, -basinLength / 2), new Vector3(basinWidth / 2, waterHeight, -basinLength / 2), new Vector3(-basinWidth / 2, waterHeight, basinLength / 2) });
            tris.Add(new Vector3[] { new Vector3(-basinWidth / 2, waterHeight, basinLength / 2), new Vector3(basinWidth / 2, waterHeight, -basinLength / 2), new Vector3(basinWidth / 2, waterHeight, basinLength / 2) });
            FluidVolume fluid = new FluidVolume(new Vector3(0, waterHeight, 0), Vector3.Up, tris, waterHeight, .75f, 8, .7f, .7f);


            //fluid.flowDirection = Vector3.Right;
            //fluid.flowForce = 80;
            //fluid.maxFlowSpeed = 50;
            space.add(fluid);
            game.modelDrawer.add(fluid);
            //Create the container.
            space.add(new Box(new Vector3(0, 0, 0), basinWidth, 1, basinLength));
            space.add(new Box(new Vector3(-basinWidth / 2 - .5f, basinHeight / 2 - .5f, 0), 1, basinHeight, basinLength));
            space.add(new Box(new Vector3(basinWidth / 2 + .5f, basinHeight / 2 - .5f, 0), 1, basinHeight, basinLength));
            space.add(new Box(new Vector3(0, basinHeight / 2 - .5f, -basinLength / 2 - .5f), basinWidth + 2, basinHeight, 1));
            space.add(new Box(new Vector3(0, basinHeight / 2 - .5f, basinLength / 2 + .5f), basinWidth + 2, basinHeight, 1));
            Random random = new Random();

            //Create a bunch of random blocks.
            /*for (int k = 0; k < 1; k++)
            {
                toAddBox = new Box(new Vector3(random.Next((int)basinWidth) - basinWidth / 2f, 30 + (.1f) * k, random.Next((int)basinLength) - basinLength / 2f), 2, 4, 2, 12);
                toAddBox.collisionMargin = .2f;
                toAddBox.allowedPenetration = .1f;
                toAddBox.orientationQuaternion = Quaternion.Normalize(new Quaternion((float)random.NextDouble(), (float)random.NextDouble(), (float)random.NextDouble(), (float)random.NextDouble()));
                space.add(toAddBox);
            }*/

            //Create a tiled floor.
            Entity toAdd;
            for (int i = 0; i < 9; i++)
            {
                for (int j = 0; j < 1; j++)
                {
                    for (int k = 0; k < 9; k++)
                    {
                        toAdd = new Box(new Vector3(-basinWidth / 2f + 10.1f * (i + 1), 15 + .01f * j, -basinLength / 2f + 10.1f * (k + 1)), 10, 5, 10, 300);
                        space.add(toAdd);
                    }
                }
            }

            //Velocity clamping is the speed under which the entity's velocity will be set to zero for
            //deactivation purposes.
            //Big floating blocks tend to have very low angular velocities.
            //To prevent strange velocity clamping behaviors, reduce the angular velocity clamping.
            space.simulationSettings.deactivation.angularVelocityClamping = .01f;

            //Create a bunch o' spheres and dump them into the water.
            /*for (int k = 0; k < 80; k++)
            {
                toAddSphere = new Sphere(new Vector3(-48 + k * 1f, 12 + 4 * k, (float)random.Next(-15, 15)), 2, 27);
                space.add(toAddSphere);
            }*/


            game.camera.position = new Vector3(0, waterHeight + 5, 15);
        }

    }
}
