﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Ring-shaped structure made of blocks.
    /// </summary>
    public class ColosseumDemo : StandardDemo
    {
        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Colosseum"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public ColosseumDemo(DemosGame game)
            : base(game)
        {
            double angle;
            int numBoxesPerRing = 12;
            float blockWidth = 2;
            float blockHeight = 2;
            float blockLength = 6f;
            float radius = 15;
            float ySpacing = blockHeight + space.simulationSettings.collisionDetection.defaultMargin * 1.99f;
            Entity toAdd;
            space.add(new Box(new Vector3(0, -ySpacing / 2 - 1, 0), 100, 2f, 100));
            double increment = MathHelper.TwoPi / numBoxesPerRing;
            for (int i = 0; i < 8; i++)
            {
                for (int k = 0; k < numBoxesPerRing; k++)
                {
                    if (i % 2 == 0)
                    {
                        angle = k * increment;
                        toAdd = new Box(new Vector3(-(float)Math.Cos(angle) * radius, i * ySpacing, (float)Math.Sin(angle) * radius), blockWidth, blockHeight, blockLength, 20);
                        toAdd.orientationQuaternion = Quaternion.CreateFromAxisAngle(Vector3.Up, (float)angle);
                        space.add(toAdd);
                    }
                    else
                    {
                        angle = (k + .5f) * increment;
                        toAdd = new Box(new Vector3(-(float)Math.Cos(angle) * radius, i * ySpacing, (float)Math.Sin(angle) * radius), blockWidth, blockHeight, blockLength, 20);
                        toAdd.orientationQuaternion = Quaternion.CreateFromAxisAngle(Vector3.Up, (float)angle);
                        space.add(toAdd);
                    }
                }
            }
            game.camera.position = new Vector3(0, 2, 2);
        }

    }
}
