﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;
using BEPUphysics;
using BEPUphysics.DataStructures;
using Microsoft.Xna.Framework.Graphics;
using BEPUphysicsDrawer.Models;
using BEPUphysics.Events;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Box changes colors as it interacts with a detector volume.
    /// </summary>
    public class DetectorVolumeDemo : StandardDemo
    {

        //Box to change colors when it hits the detector volume.
        ModelDisplayObjectBase displayBox;

        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Detector Volume"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public DetectorVolumeDemo(DemosGame game)
            : base(game)
        {
            Model tubeModel = game.Content.Load<Model>("tube");
            TriangleMeshVertex[] modelVertices;
            int[] modelIndices;
            TriangleMesh.getVerticesAndIndicesFromModel(tubeModel, out modelVertices, out modelIndices);
            DetectorVolume detectorVolume = new DetectorVolume(new TriangleMesh(modelVertices, modelIndices));
            space.add(detectorVolume);

            game.modelDrawer.add(detectorVolume.triangleMesh);

            Entity toAdd = new Box(new Vector3(0, -10, 0), 1, 1, 1);
            toAdd.tag = "noDisplayObject";
            displayBox = game.modelDrawer.add(toAdd);
            displayBox.textureIndex = 0;
            game.modelDrawer.isWireframe = true;
            toAdd.linearVelocity = new Vector3(0, 1, 0);
            space.add(toAdd); //Create the ground

            detectorVolume.eventEntityBeginsTouchingVolume += new EventHandlerEntityBeginsTouchingVolume(detectorVolume_eventEntityBeginsTouchingVolume);
            detectorVolume.eventEntityStopsTouchingVolume += new EventHandlerEntityStopsTouchingVolume(detectorVolume_eventEntityStopsTouchingVolume);
            detectorVolume.eventVolumeBeginsContainingEntity += new EventHandlerVolumeBeginsContainingEntity(detectorVolume_eventVolumeBeginsContainingEntity);
            detectorVolume.eventVolumeStopsContainingEntity += new EventHandlerVolumeStopsContainingEntity(detectorVolume_eventVolumeStopsContainingEntity);

            game.camera.position = new Vector3(0, 0, 22);
        }

        void detectorVolume_eventVolumeStopsContainingEntity(DetectorVolume volume, Entity entity)
        {
            displayBox.textureIndex = 3;
        }

        void detectorVolume_eventVolumeBeginsContainingEntity(DetectorVolume volume, Entity entity)
        {
            displayBox.textureIndex = 2;
        }

        void detectorVolume_eventEntityStopsTouchingVolume(Entity toucher, DetectorVolume volume)
        {
            displayBox.textureIndex = 0;
        }

        void detectorVolume_eventEntityBeginsTouchingVolume(Entity toucher, DetectorVolume volume)
        {
            displayBox.textureIndex = 1;
        }

        public override void cleanUp()
        {
            game.modelDrawer.isWireframe = false;
            base.cleanUp();
        }
    }
}
