﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;
using BEPUphysics.Constraints;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Earthquake simulator with a jenga stack test subject.
    /// </summary>
    public class EarthquakeDemo : StandardDemo
    {
        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Earthquake!"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public EarthquakeDemo(DemosGame game)
            : base(game)
        {
            Entity ground = new Box(new Vector3(0, 0, 0), 50f, 1f, 50f);
            space.add(ground);
            //Springs: Create a lattice of springs to hold the boxes together.
            Box platform = new Box(new Vector3(0, 4, 0), 18, 1, 18, 400);
            platform.dynamicFriction = .8f;

            space.add(platform);

            //Create a big mess of DistanceJoints representing springs.

            DistanceJoint distanceJoint = new DistanceJoint(ground, platform, new Vector3(9, .5f, -9), new Vector3(9, 3.5f, -9));
            distanceJoint.springSettings.stiffnessConstant = 3000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);
            distanceJoint = new DistanceJoint(ground, platform, new Vector3(9, .5f, -9), new Vector3(-9, 3.5f, -9));
            distanceJoint.springSettings.stiffnessConstant = 3000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);
            distanceJoint = new DistanceJoint(ground, platform, new Vector3(-9, .5f, -9), new Vector3(9, 3.5f, -9));
            distanceJoint.springSettings.stiffnessConstant = 3000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);
            distanceJoint = new DistanceJoint(ground, platform, new Vector3(-9, .5f, -9), new Vector3(-9, 3.5f, -9));
            distanceJoint.springSettings.stiffnessConstant = 3000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);

            distanceJoint = new DistanceJoint(ground, platform, new Vector3(9, .5f, 9), new Vector3(9, 3.5f, 9));
            distanceJoint.springSettings.stiffnessConstant = 6000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);
            distanceJoint = new DistanceJoint(ground, platform, new Vector3(9, .5f, 9), new Vector3(-9, 3.5f, 9));
            distanceJoint.springSettings.stiffnessConstant = 6000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);
            distanceJoint = new DistanceJoint(ground, platform, new Vector3(-9, .5f, 9), new Vector3(9, 3.5f, 9));
            distanceJoint.springSettings.stiffnessConstant = 6000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);
            distanceJoint = new DistanceJoint(ground, platform, new Vector3(-9, .5f, 9), new Vector3(-9, 3.5f, 9));
            distanceJoint.springSettings.stiffnessConstant = 6000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);

            distanceJoint = new DistanceJoint(ground, platform, new Vector3(9, .5f, -9), new Vector3(9, 3.5f, -9));
            distanceJoint.springSettings.stiffnessConstant = 6000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);
            distanceJoint = new DistanceJoint(ground, platform, new Vector3(9, .5f, -9), new Vector3(9, 3.5f, 9));
            distanceJoint.springSettings.stiffnessConstant = 6000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);
            distanceJoint = new DistanceJoint(ground, platform, new Vector3(9, .5f, 9), new Vector3(9, 3.5f, -9));
            distanceJoint.springSettings.stiffnessConstant = 6000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);
            distanceJoint = new DistanceJoint(ground, platform, new Vector3(9, .5f, 9), new Vector3(9, 3.5f, 9));
            distanceJoint.springSettings.stiffnessConstant = 6000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);

            distanceJoint = new DistanceJoint(ground, platform, new Vector3(-9, .5f, -9), new Vector3(-9, 3.5f, -9));
            distanceJoint.springSettings.stiffnessConstant = 6000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);
            distanceJoint = new DistanceJoint(ground, platform, new Vector3(-9, .5f, -9), new Vector3(-9, 3.5f, 9));
            distanceJoint.springSettings.stiffnessConstant = 6000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);
            distanceJoint = new DistanceJoint(ground, platform, new Vector3(-9, .5f, 9), new Vector3(-9, 3.5f, -9));
            distanceJoint.springSettings.stiffnessConstant = 6000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);
            distanceJoint = new DistanceJoint(ground, platform, new Vector3(-9, .5f, 9), new Vector3(-9, 3.5f, 9));
            distanceJoint.springSettings.stiffnessConstant = 6000;
            distanceJoint.springSettings.dampingConstant = 0;
            space.add(distanceJoint);

            int numBlocksTall = 10; //How many 'stories' tall.
            float blockWidth = 4; //Total width/length of the tower.
            float blockHeight = 1.333f;
            Entity toAdd;
            for (int i = 0; i < numBlocksTall; i++)
            {
                if (i % 2 == 0)
                {
                    for (int j = 0; j < 3; j++)
                    {
                        toAdd = new Box(new Vector3(j * (blockWidth / 3 + space.simulationSettings.collisionDetection.defaultMargin * 2) - blockWidth / 3f - space.simulationSettings.collisionDetection.defaultMargin * 2, 5 + blockHeight / 2 + space.simulationSettings.collisionDetection.defaultMargin * 2 + i * (blockHeight + space.simulationSettings.collisionDetection.defaultMargin * 2), 0), blockWidth / 3, blockHeight, blockWidth, 3 * numBlocksTall + 1 - 2 * i);
                        space.add(toAdd);
                        toAdd.dynamicFriction = .8f;
                    }
                }
                else
                {
                    for (int j = 0; j < 3; j++)
                    {
                        toAdd = new Box(new Vector3(0, 5 + blockHeight / 2 + space.simulationSettings.collisionDetection.defaultMargin * 2 + (i) * (blockHeight + space.simulationSettings.collisionDetection.defaultMargin * 2), j * (blockWidth / 3 + space.simulationSettings.collisionDetection.defaultMargin * 2) - blockWidth / 3f - space.simulationSettings.collisionDetection.defaultMargin * 2), blockWidth, blockHeight, blockWidth / 3, 3 * numBlocksTall + 1 - 2 * i);
                        space.add(toAdd);
                        toAdd.dynamicFriction = .8f;
                    }
                }
            }

            game.camera.position = new Vector3(0, 7, 30);
        }

    }
}
