﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;
using BEPUphysics;
using BEPUphysics.DataStructures;
using Microsoft.Xna.Framework.Graphics;
using BEPUphysicsDrawer.Models;
using BEPUphysics.Events;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Put the fish in a barrel, and a box moves.
    /// Game of the year!
    /// </summary>
    public class FishInABarrelDemo : StandardDemo
    {
        //Some entities to keep track of for the demo's events.
        Entity detector;
        Entity acceptedTrigger;
        Entity movedBox;

        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Fish in a Barrel"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public FishInABarrelDemo(DemosGame game)
            : base(game)
        {
            game.camera.position = new Vector3(0, 7, 30);

            Box detector = new Box(new Vector3(0, 0, 0), 1.5f, 1.5f, 1.5f);
            detector.collisionRules.personal = CollisionRule.noResponse;
            Box acceptedTrigger = new Box(new Vector3(5, 0, 0), 1.6f, .7f, .4f, 1);

            detector.tag = "noDisplayObject";
            acceptedTrigger.tag = "noDisplayObject";
            space.add(detector);
            space.add(acceptedTrigger);

            Model fish = game.Content.Load<Model>("fish");
            game.modelDrawer.add(new DisplayEntityModelTest(acceptedTrigger, fish, game.modelDrawer));

            Model barrelAndPlatform = game.Content.Load<Model>("barrelAndPlatform");
            StaticTriangleGroup.StaticTriangleGroupVertex[] staticTriangleVertices;
            int[] staticTriangleIndices;
            StaticTriangleGroup.getVerticesAndIndicesFromModel(barrelAndPlatform, out staticTriangleVertices, out staticTriangleIndices);
            //Note that the final 'margin' parameter is optional, but can be used to specify a collision margin on triangles in the static triangle group.
            TriangleMesh mesh = new TriangleMesh(staticTriangleVertices, staticTriangleIndices, 0);
            StaticTriangleGroup fishDepositoryGroup = new StaticTriangleGroup(mesh);
            fishDepositoryGroup.collisionRules.specificEntities.Add(detector, CollisionRule.noPair);
            space.add(fishDepositoryGroup);
            game.modelDrawer.add(mesh);

            this.detector = detector;

            this.acceptedTrigger = acceptedTrigger;
            movedBox = new Box(new Vector3(-4, 5, 0), 1, 1, 1, 1);
            detector.space.add(movedBox);
            detector.eventManager.addEventHook(new EventHandlerInitialCollisionDetected(initialCollisionDetected));
            detector.eventManager.addEventHook(new EventHandlerCollisionEnded(collisionEnded));
        }




        public void initialCollisionDetected(Entity sender, Entity other, CollisionPair collisionPair)
        {
            if (other == acceptedTrigger)
            {//If the detector collided with the accepted trigger, move the box.
                movedBox.centerPosition = new Vector3(4, 5, 0);
                movedBox.orientationQuaternion = Quaternion.Identity;
                movedBox.linearVelocity = Vector3.Zero;
                movedBox.angularVelocity = Vector3.Zero;
            }
        }

        public void collisionEnded(Entity sender, Entity other, CollisionPair collisionPair)
        {
            if (other == acceptedTrigger)
            {//If the detector ceases to collide, get rid of the spawned box.
                movedBox.centerPosition = new Vector3(-4, 5, 0);
                movedBox.orientationQuaternion = Quaternion.Identity;
                movedBox.linearVelocity = Vector3.Zero;
                movedBox.angularVelocity = Vector3.Zero;
            }

        }

        public override void drawUI()
        {
            game.dataTextDrawer.draw("Put the fish in the barrel!", new Vector2(500, 50));
        }

    }
}
