﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;
using BEPUphysics.ForceFields;
using BEPUphysicsDemos.SampleCode;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Boxes fall on a planetoid.
    /// </summary>
    public class GravitationalFieldDemo : StandardDemo
    {
        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Gravitational Field"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public GravitationalFieldDemo(DemosGame game)
            : base(game)
        {
            space.simulationSettings.motionUpdate.gravity = Vector3.Zero;

            Sphere planet = new Sphere(new Vector3(0, 20, 0), 30);
            space.add(planet);
            
            //The effective mass of the field, assuming 6.673*10^-11 as G, would be 100,000,000,000,000 kilograms.
            GravitationalField field = new GravitationalField(new InfiniteForceFieldShape(), planet.internalCenterPosition, 6673f, 1000);
            space.add(field);

            //Drop the "meteorites" on the planet.
            Entity toAdd;
            int numColumns = 7;
            int numRows = 7;
            int numHigh = 7;
            float separation = 3;
            for (int i = 0; i < numRows; i++)
                for (int j = 0; j < numColumns; j++)
                    for (int k = 0; k < numHigh; k++)
                    {


                        toAdd = new Box(new Vector3(separation * i - numRows * separation / 2, 70 + k * separation, separation * j - numColumns * separation / 2), 1f, 1f, 1f, 5);
                        toAdd.linearVelocity = new Vector3(10, 0, 0);
                        space.add(toAdd);

                    }
            game.camera.position = new Vector3(0, 20, 100);
        }

    }
}
