﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;
using BEPUphysics.Constraints;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// A two-body pendulum hits a stack of boxes.
    /// </summary>
    public class MultipendulumDemo : StandardDemo
    {
        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Multipendulum"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public MultipendulumDemo(DemosGame game)
            : base(game)
        {
            //Create the blocks in the pendulum.
            Box link1 = new Box(new Vector3(3, 5, 0), 1.1f, 1.1f, 1.1f, 2);
            Box link2 = new Box(new Vector3(6, 5, 0), 1.1f, 1.1f, 1.1f, 2);
            space.add(link1);
            space.add(link2);
            Box ground = new Box(new Vector3(3, -3, 0), 20, 1, 10);
            space.add(ground);
            //Connect them together.
            space.add(new BallSocketJoint(link1, link2, (link1.centerPosition + link2.centerPosition) / 2));
            space.add(new BallSocketJoint(link1, ground, new Vector3(0, 5, 0)));
            Entity toAdd;
            //Create a target stack.
            for (int k = 0; k < 4; k++)
            {
                toAdd = new Box(new Vector3(-2f, -1f + 2 * k, 0), 1.5f, 1.5f, 1.5f, 1);
                space.add(toAdd);
            }
            game.camera.position = new Vector3(0, 2, 20);
        }

    }
}
