﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;
using BEPUphysics;
using BEPUphysics.DataStructures;
using Microsoft.Xna.Framework.Graphics;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// A nice driveble landscape.
    /// </summary>
    public class PlaygroundDemo : StandardDemo
    {
        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Playground"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public PlaygroundDemo(DemosGame game)
            : base(game)
        {
            //Load in mesh data and create the group.
            StaticTriangleGroup.StaticTriangleGroupVertex[] staticTriangleVertices;
            int[] staticTriangleIndices;

            Model playgroundModel = game.Content.Load<Model>("playground");          
            //This load method wraps the TriangleMesh.getVerticesAndIndicesFromModel method 
            //to output vertices of type StaticTriangleGroupVertex instead of TriangleMeshVertex or simply Vector3.
            StaticTriangleGroup.getVerticesAndIndicesFromModel(playgroundModel, out staticTriangleVertices, out staticTriangleIndices);
            TriangleMesh mesh = new TriangleMesh(staticTriangleVertices, staticTriangleIndices, 0);
            StaticTriangleGroup group = new StaticTriangleGroup(mesh);

            space.add(group);
            //Modify the group's world matrix to rotate it around (or transform it in general).
            group.worldMatrix = Matrix.CreateFromYawPitchRoll((float)Math.PI, 0, 0) * Matrix.CreateTranslation(0, -10, 0);
            //Dump some boxes on top of it for fun.
            int numColumns = 8;
            int numRows = 8;
            int numHigh = 1;
            float separation = 10;
            Entity toAdd;
            for (int i = 0; i < numRows; i++)
                for (int j = 0; j < numColumns; j++)
                    for (int k = 0; k < numHigh; k++)
                    {


                        toAdd = new Box(new Vector3(separation * i - numRows * separation / 2, 40.00f + k * separation, separation * j - numColumns * separation / 2), 1f, 1f, 1f, 15);
                        space.add(toAdd);
                    }

            game.modelDrawer.add(group.triangleMesh);
            game.camera.position = new Vector3(0, 10, 40);
        }

    }
}
