﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Spheres bounce their way down through a maze of cylinders.
    /// </summary>
    public class PlinkoDemo : StandardDemo
    {
        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Plinko"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public PlinkoDemo(DemosGame game)
            : base(game)
        {
            //Drops a bunch of spheres down a plinko-style machine.


            space.simulationSettings.motionUpdate.gravity = new Vector3(0, -4, 10);
            //Create the container.
            space.add(new Box(new Vector3(0, -.5f, 0), 21f, 1f, 50));
            space.add(new Box(new Vector3(-10, .5f, 0), 1f, 1f, 50));
            space.add(new Box(new Vector3(10, .5f, 0), 1f, 1f, 50));
            space.add(new Box(new Vector3(0, .5f, -24.5f), 19f, 1f, 1));
            space.add(new Box(new Vector3(0, .5f, 24.5f), 19f, 1f, 1));

            //Create some obstacles.
            for (int i = 0; i < 4; i++)
            {
                for (int j = 0; j < 8; j++)
                {
                    space.add(new Cylinder(new Vector3(-7.5f + i * 5, .5f, -20 + j * 6), 1f, .8f));
                }
            }
            //Create some more obstacles, offset a bit.
            for (int i = 0; i < 3; i++)
            {
                for (int j = 0; j < 7; j++)
                {
                    space.add(new Cylinder(new Vector3(-5f + i * 5, .5f, -17 + j * 6), 1f, .8f));
                }
            }
            //Create the spheres.
            Entity toAdd;
            for (int i = 0; i < 15; i++)
            {
                toAdd = new Sphere(new Vector3(-7f + i * 1, .5f, -22), .3f, .5f);
                space.add(toAdd);
                toAdd = new Sphere(new Vector3(-7f + i * 1, .5f, -20), .3f, .5f);
                space.add(toAdd);

            }
            game.camera.position = new Vector3(0, 25, 25);
            game.camera.pitch = (float)Math.PI / -4;
        }

    }
}
