﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Bunch of blocks arranged in a pyramid, waiting to be blown up.
    /// </summary>
    public class PyramidDemo : StandardDemo
    {
        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Pyramid"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public PyramidDemo(DemosGame game)
            : base(game)
        {
            float boxSize = 2f;
            int boxCount = 20;
            space.add(new Box(new Vector3(0, -.5f, 0), boxCount * (boxSize + space.simulationSettings.collisionDetection.defaultMargin * 2) + 20, 1, Math.Min(50, boxCount * (boxSize + space.simulationSettings.collisionDetection.defaultMargin * 2))));
            
            for (int i = 0; i < boxCount; i++)
            {
                for (int j = 0; j < boxCount - i; j++)
                {
                    space.add(new Box(
                        new Vector3(-((boxCount - 1) * space.simulationSettings.collisionDetection.defaultMargin / 2 + boxCount * boxSize / 2) + (boxSize / 2 + space.simulationSettings.collisionDetection.defaultMargin) * i + j * (boxSize + space.simulationSettings.collisionDetection.defaultMargin * 2),
                                    ((boxSize / 2) + space.simulationSettings.collisionDetection.defaultMargin * 1) + i * (boxSize + space.simulationSettings.collisionDetection.defaultMargin * 1.999f),
                                    0),
                              boxSize, boxSize, boxSize, 20));
                }
            }
            //Down here are the 'destructors' used to blow up the pyramid.  One set is physically simulated, the other kinematic.

            //Sphere pow = new Sphere(new Vector3(-25, 5, 70), 2, 40);
            //pow.linearVelocity = new Vector3(0, 10, -100);
            //space.add(pow);
            //pow = new Sphere(new Vector3(-15, 10, 70), 2, 40);
            //pow.linearVelocity = new Vector3(0, 10, -100);
            //space.add(pow);
            //pow = new Sphere(new Vector3(-5, 15, 70), 2, 40);
            //pow.linearVelocity = new Vector3(0, 10, -100);
            //space.add(pow);
            //pow = new Sphere(new Vector3(5, 15, 70), 2, 40);
            //pow.linearVelocity = new Vector3(0, 10, -100);
            //space.add(pow);
            //pow = new Sphere(new Vector3(15, 10, 70), 2, 40);
            //pow.linearVelocity = new Vector3(0, 10, -100);
            //space.add(pow);
            //pow = new Sphere(new Vector3(25, 5, 70), 2, 40);
            //pow.linearVelocity = new Vector3(0, 10, -100);
            //space.add(pow);


            //staticSphere = new Sphere(new Vector3(-40, 40, 120), 4);
            //staticSphere.linearVelocity = new Vector3(0, 0, -70);
            //space.add(staticSphere);
            //staticSphere = new Sphere(new Vector3(-20, 40, 120), 9);
            //staticSphere.linearVelocity = new Vector3(0, 0, -85);
            //space.add(staticSphere);
            //staticSphere = new Sphere(new Vector3(0, 40, 120), 14);
            //staticSphere.linearVelocity = new Vector3(0, 0, -100);
            //space.add(staticSphere);
            //staticSphere = new Sphere(new Vector3(20, 40, 120), 9);
            //staticSphere.linearVelocity = new Vector3(0, 0, -85);
            //space.add(staticSphere);
            //staticSphere = new Sphere(new Vector3(40, 40, 120), 4);
            //staticSphere.linearVelocity = new Vector3(0, 0, -70);
            //space.add(staticSphere);


            game.camera.position = new Vector3(-boxCount * boxSize, 2, boxCount * boxSize);
            game.camera.yaw = (float)Math.PI / -4f;
            game.camera.pitch = (float)Math.PI / 9f;
        }

    }
}
