﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// A large number of blocks fall from the sky into stacks and deactivate.
    /// </summary>
    public class SleepModeDemo : StandardDemo
    {
        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Sleep Mode Stress Test"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public SleepModeDemo(DemosGame game)
            : base(game)
        {
            //Create a bunch of blocks.
            int zOffset = 5;
            int numRows = 1;
            int numColumns = 40;
            Entity toAdd;
            for (int i = 0; i < numRows; i++)
            {
                for (int j = 0; j < numColumns; j++)
                {
                    for (int k = 1; k <= 7; k++)
                    {
                        if (k % 2 == 1)
                        {
                            toAdd = new Box(new Vector3(j * 10 + -3, j * 10 + 3 * k, i * 10 + zOffset), 1, 1, 7, 20);
                            toAdd.linearDamping = .9f;
                            toAdd.angularDamping = .9f;
                            space.add(toAdd);
                            toAdd = new Box(new Vector3(j * 10 + 3, j * 10 + 3 * k, i * 10 + zOffset), 1, 1, 7, 20);
                            toAdd.linearDamping = .9f;
                            toAdd.angularDamping = .9f;
                            space.add(toAdd);
                        }
                        else
                        {
                            toAdd = new Box(new Vector3(j * 10 + 0, j * 10 + 3 * k, i * 10 + zOffset - 3), 7, 1, 1, 20);
                            toAdd.linearDamping = .9f;
                            toAdd.angularDamping = .9f;
                            space.add(toAdd);
                            toAdd = new Box(new Vector3(j * 10 + 0, j * 10 + 3 * k, i * 10 + zOffset + 3), 7, 1, 1, 20);
                            toAdd.linearDamping = .9f;
                            toAdd.angularDamping = .9f;
                            space.add(toAdd);
                        }
                    }
                    space.add(new Box(new Vector3(10 * j, -.5f, i * 10 + zOffset), 10, 1f, 10));
                }
            }

            game.camera.position = new Vector3(-30, 5, 25);
            game.camera.yaw = (float)Math.PI / -3;
            game.camera.pitch = -(float)Math.PI / -12;
        }

    }
}
