﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;
using BEPUphysics;
using BEPUphysics.ForceFields;
using BEPUphysicsDemos.SampleCode;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Unsuspecting blocks get ambushed by a whirlwind.
    /// </summary>
    public class TornadoDemo : StandardDemo
    {
        Tornado tornado;
        BoundingBoxForceFieldShape shape;

        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Tornado"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public TornadoDemo(DemosGame game)
            : base(game)
        {
            shape = new BoundingBoxForceFieldShape(new BoundingBox(new Vector3(-100, -20, -40), new Vector3(-20, 120, 40)));
            tornado = new Tornado(shape, (shape.boundingBox.Min + shape.boundingBox.Max) / 2, new Vector3(0, 1, 0), 
                                          150, false, 50, 10, 200, 200, 80, 2000, 40, 10);
            tornado.forceWakeUp = true; //The tornado will be moving, so it should wake up things that it comes into contact with.
            space.add(tornado);

            //Create the unfortunate box-like citizens about to be hit by the tornado.
            int numColumns = 10;
            int numRows = 10;
            int numHigh = 1;
            float separation = 1.5f;
            Entity toAdd;
            for (int i = 0; i < numRows; i++)
                for (int j = 0; j < numColumns; j++)
                    for (int k = 0; k < numHigh; k++)
                    {


                        toAdd = new Box(new Vector3(
                            separation * i - numRows * separation / 2,
                            5 + k * separation,
                            separation * j - numColumns * separation / 2),
                            1, 1, 1, 10);
                        space.add(toAdd);


                    }

            //x and y, in terms of heightmaps, refer to their local x and y coordinates.  In world space, they correspond to x and z.
            //Setup the heights of the terrain.
            int xLength = 256;
            int yLength = 256;

            float xSpacing = 8f;
            float ySpacing = 8f;
            float[,] heights = new float[xLength, yLength];
            float x, y;
            for (int i = 0; i < xLength; i++)
            {
                for (int j = 0; j < yLength; j++)
                {
                    x = i - xLength / 2;
                    y = j - yLength / 2;
                    //heights[i, j] = (float)Math.Pow(1.2 * Math.Sqrt(x * x + y * y), 2);
                    //heights[i, j] = -1f / (x * x + y * y);
                    //heights[i, j] = (float)(x * y / 100f);
                    heights[i, j] = (float)(5 * (Math.Sin(x / 8f) + Math.Sin(y / 8f)));
                    //heights[i, j] = 3 * (float)Math.Sin(x * y / 100f);
                    //heights[i, j] = (x * x * x * y - y * y * y * x) / 1000f;

                }
            }

            //Create the terrain.
            Terrain terrain = new Terrain(new Vector3(-xLength * xSpacing / 2, 0, -yLength * ySpacing / 2));
            terrain.setData(heights, QuadFormats.lowerLeftUpperRight, xSpacing, ySpacing);
            space.add(terrain);
            game.modelDrawer.add(terrain);
            game.camera.position = new Vector3(0, 5, 60);
        }

        public override void update(float dt)
        {
            //Move the origin of the force of the tornado,
            Vector3 increment = new Vector3(10,0,0) * dt;
            //Move the detection shape as well.
            shape.boundingBox = new BoundingBox(shape.boundingBox.Min + increment, shape.boundingBox.Max + increment);
            tornado.position += increment;
            base.update(dt);
        }
    }
}
