﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// A variety of shapes interact, and a sphere passes by at high speeds.
    /// </summary>
    public class VariousThingsDemo : StandardDemo
    {
        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Various Things Doing Stuff"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public VariousThingsDemo(DemosGame game)
            : base(game)
        {
            Entity toAdd;
            //Create a stack of spheres to sit atop the cone.
            for (int k = 0; k < 15; k++)
            {
                toAdd = new Sphere(new Vector3(0, 2 * k + 14, 0f), .5f, 1f);
                space.add(toAdd);
            }
            //Add the supporting, floaty cone.  Note the high linear damping.
            toAdd = new Cone(new Vector3(0, 12, 0), 2f, 2f, 100);
            toAdd.linearDamping = .9999f;
            space.add(toAdd);
            toAdd = new Cylinder(new Vector3(2, 70, 0), 4f, 1.2f, 100);
            toAdd.linearDamping = .73f;
            toAdd.angularDamping = .4f;
            space.add(toAdd);
            space.add(new Box(new Vector3(0, -.5f, 0), 30, 1f, 30));
            //Now for some random, circularly placed capsules to complete the scene.
            for (double k = 0; k < Math.PI * 2; k += Math.PI / 6)
            {
                space.add(new Capsule(new Vector3((float)Math.Cos(k) * 7, 2, (float)Math.Sin(k) * 7), 2, 1, 20));
            }
            for (double k = 0; k < Math.PI * 2; k += Math.PI / 16)
            {
                space.add(new Capsule(new Vector3((float)Math.Cos(k) * 12, 1, (float)Math.Sin(k) * 12), 2, 1));
            }
            //Toss a sphere from the left to knock the spherestack over.
            //Note that if the velocity of the sphere is increased, using the default setting of useContinuousDetectionAgainstKinematics = false,
            //the collision can be missed.  It is set to false by default to improve the appearance of simulations; 
            //infinite mass bodies stopping (cutting motion off to create a contact point) for less than infinite mass bodies can look a bit odd.
            Sphere staticSphere = new Sphere(new Vector3(-240, 14, 0), 2);
            staticSphere.linearVelocity = new Vector3(100, 0, 0);
            space.add(staticSphere);

            //Long poles benefit from having a more accurate model of angular momentum.  The following methods are slightly slower than the default nonconservative/symplectic euler methods.
            space.simulationSettings.motionUpdate.conserveAngularMomentum = true;
            space.simulationSettings.motionUpdate.useRK4AngularIntegration = true; //turning off RK4 while conservation is on will make objects tend to spin around the axis of least resistance in an odd way.
            //Create some long poles.
            for (int k = 0; k < 5; k++)
            {
                space.add(new Box(new Vector3(0, 6 + 2 * k, 7), 10, .2f, .2f, 20f));
            }


            game.camera.position = new Vector3(0, 6, 30);
        }

    }
}
