﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Demo showing a wall of blocks stacked up.
    /// </summary>
    public class WallDemo : StandardDemo
    {
        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string name
        {
            get { return "Wall"; }
        }

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public WallDemo(DemosGame game)
            : base(game)
        {
            int width = 10;
            int height = 10;
            float blockWidth = 2f;
            float blockHeight = 1f;
            float blockLength = 1f;
            float xSpacing = blockWidth + space.simulationSettings.collisionDetection.defaultMargin * 2;
            float ySpacing = blockHeight + space.simulationSettings.collisionDetection.defaultMargin * 2;

            Entity toAdd;

            for (int i = 0; i < width; i++)
            {
                for (int j = 0; j < height; j++)
                {
                    toAdd = new Box(new Vector3(i * xSpacing + .5f * blockWidth * (j % 2) - width * xSpacing * .5f, blockHeight * .5f + space.simulationSettings.collisionDetection.defaultMargin * 2 + j * ySpacing, 0), blockWidth, blockHeight, blockLength, 10);
                    space.add(toAdd);
                }
            }

            space.add(new Box(new Vector3(0, -.5f, 0), 50f, 1, 50f));
            game.camera.position = new Vector3(0, 6, 15);
        }

    }
}
