﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */


using BEPUphysics.Constraints;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using System;

namespace BEPUphysicsDrawer.Lines
{
    /// <summary>
    /// Graphical representation of a twist joint
    /// </summary>
    public class DisplayEllipseSwingLimit : LineDisplayObject<EllipseSwingLimit>
    {
        /// <summary>
        /// Number of facets to use when representing the swing limit boundary.
        /// </summary>
        static int limitFacetCount = 32;

        Line axis;
        Line[] limitLines;


        public DisplayEllipseSwingLimit(EllipseSwingLimit constraint, LineDrawer drawer)
            : base(drawer, constraint)
        {

            axis = new Line(Color.Red, Color.Red, drawer);
            myLines.Add(axis);
            //Create the lines that represent the outline of the limit.
            limitLines = new Line[limitFacetCount * 2];
            for (int i = 0; i < limitFacetCount; i++)
            {
                limitLines[i * 2] = new Line(Color.DarkRed, Color.DarkRed, drawer);
                limitLines[i * 2 + 1] = new Line(Color.DarkRed, Color.DarkRed, drawer);
            }
            myLines.AddRange(limitLines);

        }


        /// <summary>
        /// Moves the constraint lines to the proper location relative to the entities involved.
        /// </summary>
        public override void update()
        {

            //Move lines around
            axis.positionA = lineObject.connectionB.centerOfMass;
            axis.positionB = lineObject.connectionB.centerOfMass + lineObject.twistAxisB * 1.5f;


            float angleIncrement = 4 * MathHelper.Pi / limitLines.Length; //Each loop iteration moves this many radians forward.
            for (int i = 0; i < limitLines.Length / 2; i++)
            {
                Line pointToPreviousPoint = limitLines[2 * i];
                Line centerToPoint = limitLines[2 * i + 1];

                float currentAngle = i * angleIncrement;

                //Using the parametric equation for an ellipse, compute the axis of rotation and angle.
                Vector3 rotationAxis = lineObject.basis.xAxis * lineObject.maximumAngleX * (float)Math.Cos(currentAngle) + 
                                       lineObject.basis.yAxis * lineObject.maximumAngleY * (float)Math.Sin(currentAngle);
                float angle = rotationAxis.Length();
                rotationAxis /= angle;

                pointToPreviousPoint.positionA = lineObject.connectionB.centerOfMass +
                    //Rotate the primary axis to the ellipse boundary...
                    Vector3.TransformNormal(lineObject.basis.primaryAxis, Matrix.CreateFromAxisAngle(rotationAxis, angle));

                centerToPoint.positionA = pointToPreviousPoint.positionA;
                centerToPoint.positionB = lineObject.connectionB.centerOfMass;
            }
            for (int i = 0; i < limitLines.Length / 2; i++)
            {
                //Connect all the pointToPreviousPoint lines to the previous points.
                limitLines[2 * i].positionB = limitLines[2 * ((i + 1) % (limitLines.Length / 2))].positionA;
            }



        }


    }
}
