﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */


using BEPUphysics.Constraints;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace BEPUphysicsDrawer.Lines
{
    /// <summary>
    /// Graphical representation of a ball socket joint.
    /// </summary>
    public class DisplayRevoluteLimit : LineDisplayObject<RevoluteLimit>
    {
        Line topRight;
        Line top;
        Line topLeft;
        Line bottomRight;
        Line bottom;
        Line bottomLeft;
        Line middle;
        Line testAxis;

        public DisplayRevoluteLimit(RevoluteLimit constraint, LineDrawer drawer)
            : base(drawer, constraint)
        {
            topRight = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            top = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            topLeft = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            bottomRight = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            bottom = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            bottomLeft = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            middle = new Line(Color.DarkBlue, Color.DarkBlue, drawer);
            testAxis = new Line(Color.DarkRed, Color.DarkRed, drawer);

            myLines.Add(topRight);
            myLines.Add(top);
            myLines.Add(topLeft);
            myLines.Add(bottomRight);
            myLines.Add(bottom);
            myLines.Add(bottomLeft);
            myLines.Add(middle);
            myLines.Add(testAxis);
        }


        /// <summary>
        /// Moves the constraint lines to the proper location relative to the entities involved.
        /// </summary>
        public override void update()
        {

            //Move lines around
            Vector3 left = lineObject.connectionB.centerPosition - lineObject.basis.primaryAxis;
            Vector3 right = lineObject.connectionB.centerPosition + lineObject.basis.primaryAxis;

            Vector3 upwardsOffset = Vector3.TransformNormal(lineObject.basis.xAxis, Matrix.CreateFromAxisAngle(lineObject.basis.primaryAxis, lineObject.maximumAngle));
            Vector3 topRightPosition = right + upwardsOffset;
            Vector3 topLeftPosition = left + upwardsOffset;

            Vector3 downwardsOffset = Vector3.TransformNormal(lineObject.basis.xAxis, Matrix.CreateFromAxisAngle(lineObject.basis.primaryAxis, lineObject.minimumAngle));
            Vector3 bottomRightPosition = right + downwardsOffset;
            Vector3 bottomLeftPosition = left + downwardsOffset;

            middle.positionA = left;
            middle.positionB = right;

            topLeft.positionA = left;
            topLeft.positionB = topLeftPosition;

            topRight.positionA = right;
            topRight.positionB = topRightPosition;

            top.positionA = topLeftPosition;
            top.positionB = topRightPosition;

            bottomLeft.positionA = left;
            bottomLeft.positionB = bottomLeftPosition;

            bottomRight.positionA = right;
            bottomRight.positionB = bottomRightPosition;

            bottom.positionA = bottomLeftPosition;
            bottom.positionB = bottomRightPosition;

            testAxis.positionA = lineObject.connectionB.centerPosition;
            testAxis.positionB = lineObject.connectionB.centerPosition + lineObject.testAxis;



        }

    }
}
