﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace BEPUphysicsDrawer.Lines
{
    /// <summary>
    /// Represents a graphical line in the ConstraintDrawer.
    /// </summary>
    public class Line
    {
        LineDrawer drawer;
        /// <summary>
        /// First index of the line in the constraint drawer vertex list.
        /// </summary>
        public readonly int indexA;

        /// <summary>
        /// Second index of the line in the constraint drawer vertex list.
        /// </summary>
        public readonly int indexB;

        /// <summary>
        /// Gets and sets the position of the first vertex of the line.
        /// </summary>
        public Vector3 positionA
        {
            get
            {
                return drawer.vertices[indexA].Position;
            }
            set
            {
                drawer.vertices[indexA].Position = value;
            }
        }

        /// <summary>
        /// Gets and sets the position of the second vertex of the line.
        /// </summary>
        public Vector3 positionB
        {
            get
            {
                return drawer.vertices[indexB].Position;
            }
            set
            {
                drawer.vertices[indexB].Position = value;
            }
        }

        /// <summary>
        /// Gets and sets the color of the first vertex of the line.
        /// </summary>
        public Color colorA
        {
            get
            {
                return drawer.vertices[indexA].Color;
            }
            set
            {
                drawer.vertices[indexA].Color = value;
            }
        }

        /// <summary>
        /// Gets and sets the color of the second vertex of the line.
        /// </summary>
        public Color colorB
        {
            get
            {
                return drawer.vertices[indexB].Color;
            }
            set
            {
                drawer.vertices[indexB].Color = value;
            }
        }

        /// <summary>
        /// Constructs a new line.
        /// </summary>
        /// <param name="positionA">Initial position of the first vertex of the line.</param>
        /// <param name="positionB">Initial position of the second vertex of the line.</param>
        /// <param name="colorA">Initial color of the first vertex of the line.</param>
        /// <param name="colorB">Initial color of the second vertex of the line.</param>
        /// <param name="drawer">System responsible for drawing this line.</param>
        public Line(Vector3 positionA, Vector3 positionB, Color colorA, Color colorB, LineDrawer drawer)
        {
            this.drawer = drawer;
            drawer.getNewLineIndex(out indexA);
            indexB = indexA + 1;
            drawer.vertices[indexA].Position = positionA;
            drawer.vertices[indexA].Color = colorA;
            drawer.vertices[indexB].Position = positionB;
            drawer.vertices[indexB].Color = colorB;
        }

        /// <summary>
        /// Constructs a new line.
        /// </summary>
        /// <param name="colorA">Initial color of the first vertex of the line.</param>
        /// <param name="colorB">Initial color of the second vertex of the line.</param>
        /// <param name="drawer">System responsible for drawing this line.</param>
        public Line(Color colorA, Color colorB, LineDrawer drawer)
        {
            this.drawer = drawer;
            drawer.getNewLineIndex(out indexA);
            indexB = indexA + 1;
            drawer.vertices[indexA].Position = Vector3.Zero;
            drawer.vertices[indexA].Color = colorA;
            drawer.vertices[indexB].Position = Vector3.Zero;
            drawer.vertices[indexB].Color = colorB;
        }




    }
}
