﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;

namespace BEPUphysicsDrawer.Models
{
    /// <summary>
    /// Stores vertex and index data for the brute force renderer.
    /// </summary>
    internal class BruteDisplayObjectEntry
    {
        internal ModelDisplayObjectBase displayObject; 

        internal VertexPositionNormalTexture[] vertices;
        internal ushort[] indices;

        internal VertexBuffer vertexBuffer;
        internal IndexBuffer indexBuffer;

        internal BruteModelDrawer drawer;

        internal BruteDisplayObjectEntry(BruteModelDrawer drawer, ModelDisplayObjectBase displayObject)
        {
            this.drawer = drawer;
            this.displayObject = displayObject;

            List<VertexPositionNormalTexture> tempVertices = new List<VertexPositionNormalTexture>();
            List<ushort> tempIndices = new List<ushort>();
            displayObject.getVertexData(tempVertices, tempIndices);
            vertices = new VertexPositionNormalTexture[tempVertices.Count];
            indices = new ushort[tempIndices.Count];
            tempVertices.CopyTo(vertices);
            tempIndices.CopyTo(indices);

            vertexBuffer = new VertexBuffer(displayObject.drawer.game.GraphicsDevice, typeof(VertexPositionNormalTexture), vertices.Length, BufferUsage.WriteOnly);
            indexBuffer = new IndexBuffer(displayObject.drawer.game.GraphicsDevice, IndexElementSize.SixteenBits, indices.Length, BufferUsage.WriteOnly);

            vertexBuffer.SetData(vertices);
            indexBuffer.SetData(indices);
        }

        internal void draw(Texture2D[] textures, GraphicsDevice device, BasicEffect effect, EffectPass pass)
        {
            effect.World = displayObject.worldTransform;
            effect.Texture = textures[displayObject.textureIndex];
            device.SetVertexBuffer(vertexBuffer);
            device.Indices = indexBuffer;
            pass.Apply();

            device.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0, vertices.Length, 0, indices.Length / 3);
        }
    }
}
