﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using BEPUphysics.Entities;

namespace BEPUphysicsDrawer.Models
{
    /// <summary>
    /// Model which just sits in space.
    /// </summary>
    public class DisplayModelTest : SelfDrawingModelDisplayObject
    {

        Model myModel;
        /// <summary>
        /// Gets or sets the model to display.
        /// </summary>
        public Model model
        {
            get
            {
                return myModel;
            }
            set
            {
                myModel = value;
                transforms = new Matrix[myModel.Bones.Count];
                for (int i = 0; i < model.Meshes.Count; i++)
                {
                    for (int j = 0; j < model.Meshes[i].Effects.Count; j++)
                    {
                        BasicEffect effect = model.Meshes[i].Effects[j] as BasicEffect;
                        if (effect != null)
                            effect.EnableDefaultLighting();
                    }
                }
            }
        }

        Texture2D myTexture;
        /// <summary>
        /// Gets or sets the texture drawn on this model.
        /// </summary>
        public Texture2D texture
        {
            get
            {
                return myTexture;
            }
            set
            {
                myTexture = value;
                for (int i = 0; i < model.Meshes.Count; i++)
                {
                    for (int j = 0; j < model.Meshes[i].Effects.Count; j++)
                    {
                        BasicEffect effect = model.Meshes[i].Effects[j] as BasicEffect;
                        if (effect != null)
                        {
                            if (value != null)
                            {
                                effect.TextureEnabled = true;
                                effect.Texture = texture;
                            }
                            else
                                effect.TextureEnabled = false;
                        }
                    }
                }
            }
        }


        /// <summary>
        /// Gets or sets the world transformation applied to the model.
        /// </summary>
        public Matrix worldTransform { get; set; }

        /// <summary>
        /// Bone transformations of meshes in the model.
        /// </summary>
        Matrix[] transforms;

        /// <summary>
        /// Constructs a new display model.
        /// </summary>
        /// <param name="model">Model to draw on the entity.</param>
        /// <param name="modelDrawer">Model drawer to use.</param>
        public DisplayModelTest(Model model, ModelDrawer modelDrawer)
            : base(modelDrawer)
        {
            this.model = model;
        }

        /// <summary>
        /// Updates the display object.
        /// </summary>
        public override void update()
        {
        }

        /// <summary>
        /// Draws the display object.
        /// </summary>
        /// <param name="viewMatrix">Current view matrix.</param>
        /// <param name="projectionMatrix">Current projection matrix.</param>
        public override void draw(Matrix viewMatrix, Matrix projectionMatrix)
        {
            //This is not a particularly fast method of drawing.
            //It's used very rarely in the demos.
            myModel.CopyAbsoluteBoneTransformsTo(transforms);
            for (int i = 0; i < model.Meshes.Count; i++)
            {
                for (int j = 0; j < model.Meshes[i].Effects.Count; j++)
                {
                    BasicEffect effect = model.Meshes[i].Effects[j] as BasicEffect;
                    if (effect != null)
                    {
                        effect.World = transforms[model.Meshes[i].ParentBone.Index] * worldTransform;
                        effect.View = viewMatrix;
                        effect.Projection = projectionMatrix;
                    }
                }
                model.Meshes[i].Draw();
            }
        }
    }
}
