/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */


using System;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using BEPUphysics.Entities;
using System.Collections.Generic;
using System.Diagnostics;

namespace BEPUphysicsDrawer.Models
{
    /// <summary>
    /// Simple display object for triangles.
    /// </summary>
    public class DisplayCompoundBodyTest : DisplayEntity<CompoundBody>
    {

        /// <summary>
        /// Creates the display object for the entity.
        /// </summary>
        /// <param name="drawer">Drawer managing this display object.</param>
        /// <param name="displayedObject">Entity to draw.</param>
        public DisplayCompoundBodyTest(ModelDrawer drawer, CompoundBody displayedObject)
            : base(drawer, displayedObject)
        {

        }

        public override int getTriangleCountEstimate()
        {
            return 100;
        }

        public override void getVertexData(List<VertexPositionNormalTexture> vertices, List<ushort> indices)
        {
            List<ushort> tempIndices = new List<ushort>();
            List<VertexPositionNormalTexture> tempVertices = new List<VertexPositionNormalTexture>();
            for (int i = 0; i < displayedObject.subBodies.Count; i++)
            {
                ModelDisplayObjectBase displayObject = drawer.getDisplayObject(displayedObject.subBodies[i]);

                displayObject.getVertexData(tempVertices, tempIndices);
                for (int j = 0; j < tempIndices.Count; j++)
                {
                    indices.Add((ushort)(tempIndices[j] + vertices.Count));
                }
                for (int j = 0; j < tempVertices.Count; j++)
                {
                    VertexPositionNormalTexture vertex = tempVertices[j];
                    vertex.Position = Vector3.Transform(vertex.Position, displayedObject.subBodyLocalRotations[i]) + displayedObject.subBodyLocalOffsets[i];
                    vertex.Normal = Vector3.Transform(vertex.Normal, displayedObject.subBodyLocalRotations[i]);
                    vertices.Add(vertex);
                }

                tempVertices.Clear();
                tempIndices.Clear();
            }


        }

    }
}
